@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        scheduler.lua
@@ -21,6 +21,7 @@
 -- define module: scheduler
 local scheduler  = scheduler or {}
 local _coroutine = _coroutine or {}
+local _semaphore = _semaphore or {}
 
 -- load modules
 local table     = require("base/table")
@@ -30,9 +31,119 @@ local string    = require("base/string")
 local poller    = require("base/poller")
 local timer     = require("base/timer")
 local hashset   = require("base/hashset")
+local queue     = require("base/queue")
 local coroutine = require("base/coroutine")
 local bit       = require("base/bit")
 
+-- new a semaphore instance
+function _semaphore.new(name, value)
+    local instance    = table.inherit(_semaphore)
+    instance._NAME    = name
+    instance._VALUE   = value or 0
+    instance._WAITING = queue.new()
+    instance._POSTING = false
+    setmetatable(instance, _semaphore)
+    return instance
+end
+
+-- get the semaphore name
+function _semaphore:name()
+    return self._NAME or "none"
+end
+
+-- post the semaphore value
+function _semaphore:post(value)
+    if self._POSTING then
+        return self._VALUE
+    end
+    self._POSTING = true
+    local new_value = self._VALUE + value
+    self._VALUE = new_value
+    if new_value > 0 then
+        local waiting = self._WAITING
+        local post_count = 0
+        while not waiting:empty() do
+            local cp = waiting:pop()
+            if not cp:is_suspended() then
+                self._POSTING = false
+                return -1, string.format("%s cannot be resumed, status: %s", co, co:status())
+            end
+            local ok, errors = scheduler:co_resume(cp)
+            if not ok then
+                self._POSTING = false
+                return -1, errors
+            end
+            post_count = post_count + 1
+            if post_count >= new_value then
+                break
+            end
+        end
+    end
+    self._POSTING = false
+    return new_value
+end
+
+-- wait the semaphore
+function _semaphore:wait(timeout)
+
+    -- get the running coroutine
+    local running = scheduler:co_running()
+    if not running then
+        return -1, "we must call semaphore:wait() in coroutine with scheduler!"
+    end
+
+    -- is stopped?
+    if not scheduler._STARTED then
+        return -1, "the scheduler is stopped!"
+    end
+
+    -- update value
+    local value = self._VALUE
+    if value > 0 then
+        self._VALUE = value - 1
+        return value
+    end
+
+    -- no signal? return immediately if timeout is zero
+    if timeout == 0 then
+        return 0
+    end
+
+    -- wait semaphore
+    self._WAITING:push(running)
+    if timeout > 0 then
+        scheduler:_timer():post(function (cancel)
+            if running:is_suspended() then
+                return scheduler:co_resume(running, true)
+            end
+            return true
+        end, timeout)
+    end
+
+    while true do
+        local timeout = scheduler:co_suspend()
+
+        local value = self._VALUE
+        if value > 0 then
+            self._VALUE = value - 1
+            return value
+        end
+
+        if timeout then
+            break
+        end
+
+        -- continue to wait it
+        self._WAITING:push(running)
+    end
+    return 0
+end
+
+-- tostring(semaphore)
+function _semaphore:__tostring()
+    return string.format("<co_semaphore: %s/%d>", self:name(), self._VALUE)
+end
+
 -- new a coroutine instance
 function _coroutine.new(name, thread)
     local instance   = table.inherit(_coroutine)
@@ -179,8 +290,10 @@ function scheduler:_poller_resume_co(co, events)
         events = poller.EV_POLLER_ERROR
     end
 
-    -- this coroutine must be suspended
-    assert(co:is_suspended())
+    -- this coroutine must be suspended, (maybe also dead)
+    if not co:is_suspended() then
+        return false, string.format("%s cannot be resumed, status: %s", co, co:status())
+    end
 
     -- resume this coroutine task
     co:waitobj_set(nil)
@@ -271,18 +384,15 @@ function scheduler:_co_curdir_update(curdir)
         return
     end
 
-    -- save the current directory hash
+    -- save the current directory
     curdir = curdir or os.curdir()
-    local curdir_hash = hash.uuid4(path.absolute(curdir)):sub(1, 8)
-    self._CO_CURDIR_HASH = curdir_hash
-
-    -- save the current directory for each coroutine
     local co_curdirs = self._CO_CURDIRS
     if not co_curdirs then
         co_curdirs = {}
         self._CO_CURDIRS = co_curdirs
     end
-    co_curdirs[running] = {curdir_hash, curdir}
+    co_curdirs[running] = curdir
+    self._CO_CURDIR_CURRENT = curdir
 end
 
 -- update the current environments hash of current coroutine
@@ -300,7 +410,7 @@ function scheduler:_co_curenvs_update(envs)
     for _, key in ipairs(table.orderkeys(envs)) do
         envs_hash = envs_hash .. key:upper() .. envs[key]
     end
-    envs_hash = hash.uuid4(envs_hash):sub(1, 8)
+    envs_hash = hash.strhash64(envs_hash)
     self._CO_CURENVS_HASH = envs_hash
 
     -- save the current directory for each coroutine
@@ -434,7 +544,29 @@ end
 
 -- resume the given coroutine
 function scheduler:co_resume(co, ...)
-    return coroutine.resume(co:thread(), ...)
+
+    -- do resume
+    local ok, errors = coroutine.resume(co:thread(), ...)
+
+    local running = self:co_running()
+    if running then
+
+        -- has the current directory been changed? restore it
+        local curdir = self._CO_CURDIR_CURRENT
+        local olddir = self._CO_CURDIRS and self._CO_CURDIRS[running] or nil
+        if olddir and curdir ~= olddir then -- hash changed?
+            os.cd(olddir)
+        end
+
+        -- has the current environments been changed? restore it
+        local curenvs = self._CO_CURENVS_HASH
+        local oldenvs = self._CO_CURENVS and self._CO_CURENVS[running] or nil
+        if oldenvs and curenvs ~= oldenvs[1] and running:is_isolated() then -- hash changed?
+            os.setenvs(oldenvs[2])
+        end
+    end
+
+    return ok, errors
 end
 
 -- suspend the current coroutine
@@ -443,15 +575,15 @@ function scheduler:co_suspend(...)
     -- suspend it
     local results = table.pack(coroutine.yield(...))
 
-    -- Has the current directory been changed? restore it
+    -- has the current directory been changed? restore it
     local running = assert(self:co_running())
-    local curdir = self._CO_CURDIR_HASH
+    local curdir = self._CO_CURDIR_CURRENT
     local olddir = self._CO_CURDIRS and self._CO_CURDIRS[running] or nil
-    if olddir and curdir ~= olddir[1] then -- hash changed?
-        os.cd(olddir[2])
+    if olddir and curdir ~= olddir then -- hash changed?
+        os.cd(olddir)
     end
 
-    -- Has the current environments been changed? restore it
+    -- has the current environments been changed? restore it
     local curenvs = self._CO_CURENVS_HASH
     local oldenvs = self._CO_CURENVS and self._CO_CURENVS[running] or nil
     if oldenvs and curenvs ~= oldenvs[1] and running:is_isolated() then -- hash changed?
@@ -499,6 +631,87 @@ function scheduler:co_sleep(ms)
     return true
 end
 
+-- lock the current coroutine
+function scheduler:co_lock(lockname)
+
+    -- get the running coroutine
+    local running = self:co_running()
+    if not running then
+        return false, "we must call co_lock() in coroutine with scheduler!"
+    end
+
+    -- is stopped?
+    if not self._STARTED then
+        return false, "the scheduler is stopped!"
+    end
+
+    -- do lock
+    local co_locked_tasks = self._CO_LOCKED_TASKS
+    if co_locked_tasks == nil then
+        co_locked_tasks = {}
+        self._CO_LOCKED_TASKS = co_locked_tasks
+    end
+    while true do
+
+        -- try to lock it
+        if co_locked_tasks[lockname] == nil then
+            co_locked_tasks[lockname] = running
+            return true
+        -- has been locked by the current coroutine
+        elseif co_locked_tasks[lockname] == running then
+            return true
+        end
+
+        -- register timeout task to timer
+        local function timer_callback (cancel)
+            if co_locked_tasks[lockname] == nil then
+                if running:is_suspended() then
+                    return self:co_resume(running)
+                end
+            else
+                self:_timer():post(timer_callback, 500)
+            end
+            return true
+        end
+        self:_timer():post(timer_callback, 500)
+
+        -- wait
+        self:co_suspend()
+    end
+    return true
+end
+
+-- unlock the current coroutine
+function scheduler:co_unlock(lockname)
+
+    -- get the running coroutine
+    local running = self:co_running()
+    if not running then
+        return false, "we must call co_unlock() in coroutine with scheduler!"
+    end
+
+    -- is stopped?
+    if not self._STARTED then
+        return false, "the scheduler is stopped!"
+    end
+
+    -- do unlock
+    local co_locked_tasks = self._CO_LOCKED_TASKS
+    if co_locked_tasks == nil then
+        co_locked_tasks = {}
+        self._CO_LOCKED_TASKS = co_locked_tasks
+    end
+    if co_locked_tasks[lockname] == nil then
+        return false, string.format("we need to call lock(%s) first before calling unlock(%s)", lockname, lockname)
+    end
+    if co_locked_tasks[lockname] == running then
+        co_locked_tasks[lockname] = nil
+    else
+        return false, string.format("unlock(%s) is called in other %s", lockname, running)
+    end
+    return true
+end
+
 -- get the given coroutine group
 function scheduler:co_group(name)
     return self._CO_GROUPS and self._CO_GROUPS[name]
@@ -624,6 +837,11 @@ function scheduler:co_count()
     return self._CO_COUNT or 0
 end
 
+-- new a coroutine semaphore
+function scheduler:co_semaphore(name, value)
+    return _semaphore.new(name, value)
+end
+
 -- wait poller object io events, only for socket and pipe object
 function scheduler:poller_wait(obj, events, timeout)
 
@@ -970,20 +1188,16 @@ function scheduler:runloop()
                 if eventfunc then
                     ok, errors = eventfunc(self, obj, objevents)
                     if not ok then
-                        -- TODO
-                        --
                         -- This causes a direct exit from the entire runloop and
                         -- a quick escape from nested try-catch blocks and coroutines groups.
                         --
                         -- So some try-catch cannot catch these errors, such as when a build fails (in build group).
                         -- @see https://github.com/xmake-io/xmake/issues/3401
                         --
-                        -- In theory, we should handle it better. For example, if there is a group that is waiting,
-                        -- we should notify the other concurrent threads to exit quickly and then let the group concurrent threads to throw the error.
-                        -- That way the outside try-catch can continue to catch it.
+                        -- We should catch it in coroutines and re-throw it outside scheduler,
+                        -- it will avoid co_resume to get and return this error.
                         --
-                        -- But implementing it is more complicated and I haven't come up with a solution to let other concurrent processes exit quickly,
-                        -- especially if the child process is waiting and we need to notify it of the end quickly as well.
+                        -- e.g. we can see runjobs.lua implementation
                         break
                     end
                 end
